<?php

namespace Drupal\bookkeeping\Form;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a Bookkeeping form.
 */
class TransactionListFilterForm extends FormBase {

  /**
   * The account entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $accountStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('entity_type.manager')->getStorage('bookkeeping_account')
    );
  }

  /**
   * Construct the transaction filter form.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Entity\EntityStorageInterface $account_storage
   *   The account entity storage.
   */
  public function __construct(RequestStack $request_stack, EntityStorageInterface $account_storage) {
    $this->setRequestStack($request_stack);
    $this->accountStorage = $account_storage;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bookkeeping_transaction_list_filter';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $request = $this->getRequest();
    $form['#attributes']['class'][] = 'form--inline';

    $form['description'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Description'),
      '#size' => 30,
      '#default_value' => $request->query->get('description'),
    ];

    $form['batch'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Batch'),
      '#size' => 20,
      '#default_value' => $request->query->get('batch'),
    ];

    $account_id = $request->query->get('account');
    $form['account'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Account'),
      '#target_type' => 'bookkeeping_account',
      '#size' => 30,
      '#default_value' => $account_id ? $this->accountStorage->load($account_id) : NULL,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply'),
    ];

    $form['actions']['reset'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reset'),
      '#submit' => ['::resetForm'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $options = [];
    foreach (Element::children($form) as $child) {
      $type = $form[$child]['#type'] ?? FALSE;
      if (!in_array($type, ['actions', 'hidden', 'token'])) {
        $options['query'][$child] = $form_state->getValue($child);
      }
    }
    $form_state->setRedirect('<current>', [], $options);
  }

  /**
   * {@inheritdoc}
   */
  public function resetForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('<current>');
  }

}
